@@ -20,11 +20,26 @@ static char **mnt;
 static int mntcnt;
 char *progname;
 
-static void usage(void)
+static void usage(int status)
 {
-	errstr(_("Utility for syncing quotas.\nUsage:\n%s [-ug] -a | mntpoint...\n\n"), progname);
-	fprintf(stderr, _("Bugs to %s\n"), MY_EMAIL);
-	exit(1);
+	printf(_(
+"%1$s: Utility for syncing quotas.\n"
+"Usage: %1$s [-ug] mount-point...\n"
+"   or: %1$s [-ug] -a\n"
+"   or: %1$s -h | -V\n"
+"\n"
+		), progname);
+	printf(_(
+"Options:\n"
+"-u, --user     synchronize user quotas\n"
+"-g, --group    synchronize group quotas\n"
+"-a, --all      synchronize quotas for all mounted file systems\n"
+"-h, --help     display this help message and exit\n"
+"-V, --version  display version information and exit\n"
+"\n"
+		));
+	printf(_("Report bugs to <%s>.\n"), MY_EMAIL);
+	exit(status);
 }
 
 static void parse_options(int argcnt, char **argstr)
@@ -42,11 +57,12 @@ static void parse_options(int argcnt, char **argstr)
 	while ((ret = getopt_long(argcnt, argstr, "ahugV", long_opts, NULL)) != -1) {
 		switch (ret) {
 			case '?':
+				usage(EXIT_FAILURE);
 			case 'h':
-				usage();
+				usage(EXIT_SUCCESS);
 			case 'V':
 				version();
-				exit(0);
+				exit(EXIT_SUCCESS);
 			case 'u':
 				flags |= FL_USER;
 				break;
@@ -62,7 +78,7 @@ static void parse_options(int argcnt, char **argstr)
 	if ((flags & FL_ALL && optind != argcnt) ||
 	    (!(flags & FL_ALL) && optind == argcnt)) {
 		fputs(_("Bad number of arguments.\n"), stderr);
-		usage();
+		usage(EXIT_FAILURE);
 	}
 	if (!(flags & FL_ALL)) {
 		mnt = argstr + optind;
@@ -85,10 +101,12 @@ static int syncquotas(int type)
 	int i, ret = 0;
 
 	if (flags & FL_ALL) {
-		if (sync_one(type, NULL) < 0)
+		if (sync_one(type, NULL) < 0) {
 			errstr(_("%s quota sync failed: %s\n"), _(type2name(type)),
 					strerror(errno));
-		return -1;
+			ret = -1;
+		}
+		return ret;
 	}
 
 	handles = create_handle_list(mntcnt, mnt, type, fmt,
@@ -109,7 +127,7 @@ static int syncquotas(int type)
 
 int main(int argc, char **argv)
 {
-	int ret = 0;
+	int ret = EXIT_SUCCESS;
 
 	gettexton();
 	progname = basename(argv[0]);
@@ -119,9 +137,9 @@ int main(int argc, char **argv)
 
 	if (flags & FL_USER)
 		if (syncquotas(USRQUOTA))
-			ret = 1;
+			ret = EXIT_FAILURE;
 	if (flags & FL_GROUP)
 		if (syncquotas(GRPQUOTA))
-			ret = 1;
+			ret = EXIT_FAILURE;
 	return ret;
 }
